#!/usr/bin/perl -w

#$Header: /home2/cvsroot/LogTrend/StorageServer/AddSource/PostgreSQLSourceAdder.pm,v 1.5 2001/12/10 16:58:34 lsimonneau Exp $
##******************************************************************************
## Class PostgreSQLSourceAdder
##  Description  : class for adding a new source into PostgreSQL database
##  Project      : LogTrend 1.0.0.0 - Atrid Systemes
##  Author       : Sylvain Lhullier s.lhullier@atrid.fr (02.03.2001)
##******************************************************************************
#$Log: PostgreSQLSourceAdder.pm,v $
#Revision 1.5  2001/12/10 16:58:34  lsimonneau
#Use GnuPG::Interface instead of Crypt::RSA for perl 5.005 compatibility.
#
#Revision 1.4  2001/11/16 10:25:18  lsimonneau
#Major feature enhancement : Implementation of authentification with RSA signature.
#
#IMPORTANT : This StorageServer is incompatible with older version.
#
#Revision 1.3  2001/06/07 14:25:41  slhullier
#
#Passage de  unshift @INC,'..';  aux packages Logtrend::....
#
#Revision 1.2  2001/03/27 16:31:33  slhullier
#
#Mail envoye avant arret de l'agent
#
#Revision 1.1  2001/03/02 13:42:44  slhullier
#
#AddSource fonctionne.
#

use strict;
use DBI;

package LogTrend::StorageServer::AddSource::PostgreSQLSourceAdder;

use LogTrend::StorageServer::AddSource::SourceAdder;
use IO::Handle;
use GnuPG::Interface;
use POSIX qw (tmpnam);

use vars qw( @ISA );
@ISA = ("LogTrend::StorageServer::AddSource::SourceAdder");

##******************************************************************************
## Constructor  public
##  Parameters   : none
##******************************************************************************
sub new
{
  my ($classname, $db_name, $db_host, $db_port, $db_username, $db_password) = @_;
  my $self = $classname->SUPER::new( );
  bless($self, $classname);

  $self->Connect($db_name, $db_host, $db_port, $db_username, $db_password);
  
  return $self;
}

##******************************************************************************
## Function Connect  public virtual
##  Description  : starts the connection to the database
##  Parameters   : database name, host name of the database, port number of the
##                 database, database user, database user's password
##  Return value : none
##******************************************************************************
sub Connect
{
   my ($self, $db_name, $db_host, $db_port, $db_username, $db_password) = @_;
   my $param = "dbi:Pg:dbname=$db_name";

   if( !( $db_host eq "" ) )
   {
      $param .= ";host=$db_host";
   }
   if( !( $db_port eq "" ) )
   {
      $param .= ";port=$db_port";
   }

   $self->{DB_HANDLE} = DBI->connect( $param, $db_username, $db_password ) || die($!);
}

##******************************************************************************
## Function Disconnect  public virtual
##  Description  : achieves the diconnection from a database server
##  Parameters   : none
##  Return value : none
##******************************************************************************
sub Disconnect
{
   my ($self) = @_;
   $self->{DB_HANDLE}->disconnect() || die($!);
}

##******************************************************************************
## Function Create_New_Source  public virtual
##  Description  : create a new source in the database
##  Parameters   : the source's password, a reference to an hash table
##                 containing informations
##  Return value : none
##******************************************************************************
sub Create_New_Source
{
   my ($self, $src_pub_key_file, $src_info) = @_;
   my $st_handle;


   $self->check_key($src_pub_key_file);


   ##===========================================================================
   ## Table SOURCES
   ##===========================================================================
   open(F_PUBKEY, $src_pub_key_file) 
       or die("Public key file $src_pub_key_file : $!");

   ## Try to load the key 
   my @pub_key_buf = <F_PUBKEY>;

   close(F_PUBKEY);

   $st_handle = $self->{DB_HANDLE}->prepare( "INSERT INTO sources(public_key) VALUES(?)" ) || die($!);
   $st_handle->execute( "@pub_key_buf" ) || die($!);
   $st_handle->finish();
   printf "Ok, new source created";

   $st_handle = $self->{DB_HANDLE}->prepare( "SELECT MAX(id_source) FROM sources" ) || die($!);
   $st_handle->execute( ) || die($!);
   my @row = $st_handle->fetchrow_array();
   if(defined $st_handle->err) {
       die $st_handle->errstr;
   }
   if(!@row) {
       die "Internal error, there are no source in database !";
   }

   my $id_source = $row[0];
   print " (id_source=$id_source)\n";

   ##===========================================================================
   ## Table SOURCES_INFORMATIONS
   ##===========================================================================
   foreach my $description (keys %$src_info)
   {
      $st_handle = $self->{DB_HANDLE}->prepare( "INSERT INTO sources_informations (id_source, description, value) VALUES(?,?,?)" ) 
          || die($self->{DB_HANDLE}->errstr);
      $st_handle->execute( $id_source, $description, $$src_info{$description} ) || die($!);
      $st_handle->finish();
   }
}


sub check_key {
    my ($self, $src_pub_key_file) = @_;

    my $gpg_home = tmpnam;
    

    mkdir($gpg_home, 0700)
        or die "Can't create $gpg_home : $!";

    my $gpg = new GnuPG::Interface;
    $gpg->options->hash_init( armor    => 1,
                              homedir  => $gpg_home,
                              always_trust => 1);

    my $input   = IO::Handle->new();
    my $output  = IO::Handle->new();
    my $error   = IO::Handle->new();
    my $status_fh = IO::Handle->new();
    my $handles = GnuPG::Handles->new( stdin  => $input,
                                       stdout => $output,
                                       stderr => $error,
                                       status => $status_fh,
                                       );
    
    open(KEY_FILE, "$src_pub_key_file") or die "Can't open $src_pub_key_file : $!";
    my @pub_key = <KEY_FILE>;
    close(KEY_FILE);

    my $import_pid = $gpg->import_keys( handles => $handles );
    print $input "@pub_key";
    close($input);

    waitpid($import_pid, 0);

    my $stat = <$status_fh>;
    if($stat !~ /^\[GNUPG:\] IMPORTED/) {
        my @err = <$error>;
        die "Unexpected response from GnuPG : $stat\nReturned message : @err";
    }

    $stat = <$status_fh>;
    if($stat !~ /^\[GNUPG:\] IMPORT_RES 1/) {
        my @err = <$error>;
        die "Unexpected response from GnuPG : $stat\nReturned message : @err";
    }

    unlink("$gpg_home/pubring.gpg", "$gpg_home/pubring.gpg~", "$gpg_home/random_seed", "$gpg_home/secring.gpg");
    rmdir("$gpg_home");
}


sub DESTROY {
    my $self = shift;

    $self->Disconnect;
}
##******************************************************************************
1;
