#!/usr/bin/perl -w

#$Header: /home2/cvsroot/LogTrend/StorageServer/UpdateSource/PostgreSQLSourceUpdater.pm,v 1.2 2001/12/10 16:58:34 lsimonneau Exp $
##******************************************************************************
## Class PostgreSQLSourceAdder
##  Description  : class for adding a new source into PostgreSQL database
##  Project      : LogTrend 1.0.0.0 - Atrid Systemes
##  Author       : Laurent Simonneau l.simonneau@atrid.fr
##******************************************************************************
#$Log: PostgreSQLSourceUpdater.pm,v $
#Revision 1.2  2001/12/10 16:58:34  lsimonneau
#Use GnuPG::Interface instead of Crypt::RSA for perl 5.005 compatibility.
#
#Revision 1.1  2001/11/16 10:25:18  lsimonneau
#Major feature enhancement : Implementation of authentification with RSA signature.
#
#IMPORTANT : This StorageServer is incompatible with older version.
#
#Revision 1.3  2001/06/07 14:25:41  slhullier
#
#Passage de  unshift @INC,'..';  aux packages Logtrend::....
#
#Revision 1.2  2001/03/27 16:31:33  slhullier
#
#Mail envoye avant arret de l'agent
#
#Revision 1.1  2001/03/02 13:42:44  slhullier
#
#AddSource fonctionne.
#

use strict;
use DBI;

package LogTrend::StorageServer::UpdateSource::PostgreSQLSourceUpdater;

use LogTrend::StorageServer::UpdateSource::SourceUpdater;
use IO::Handle;
use GnuPG::Interface;
use POSIX qw (tmpnam);

use vars qw( @ISA );
@ISA = ("LogTrend::StorageServer::UpdateSource::SourceUpdater");

##******************************************************************************
## Constructor  public
##  Parameters   : none
##******************************************************************************
sub new
{
  my ($classname, $db_name, $db_host, $db_port, $db_username, $db_password) = @_;
  my $self = $classname->SUPER::new( );
  bless($self, $classname);

  $self->Connect($db_name, $db_host, $db_port, $db_username, $db_password);
  
  return $self;
}

##******************************************************************************
## Function Connect  public virtual
##  Description  : starts the connection to the database
##  Parameters   : database name, host name of the database, port number of the
##                 database, database user, database user's password
##  Return value : none
##******************************************************************************
sub Connect
{
   my ($self, $db_name, $db_host, $db_port, $db_username, $db_password) = @_;
   my $param = "dbi:Pg:dbname=$db_name";

   if( !( $db_host eq "" ) )
   {
      $param .= ";host=$db_host";
   }
   if( !( $db_port eq "" ) )
   {
      $param .= ";port=$db_port";
   }

   $self->{DB_HANDLE} = DBI->connect( $param, $db_username, $db_password ) || die($!);
}

##******************************************************************************
## Function Disconnect  public virtual
##  Description  : achieves the diconnection from a database server
##  Parameters   : none
##  Return value : none
##******************************************************************************
sub Disconnect
{
   my ($self) = @_;
   $self->{DB_HANDLE}->disconnect() || die($!);
}


##******************************************************************************
## Function Update_Source  public virtual
##  Description  : update a source in the database
##  Parameters   : the source's public key filename, the source id
##  Return value : none
##******************************************************************************
sub Update_Source
{
   my ($self, $src_pub_key_file, $src_id) = @_;
   my $st_handle;

   $self->check_key($src_pub_key_file);
   
   ##===========================================================================
   ## Table SOURCES
   ##===========================================================================
   open(F_PUBKEY, $src_pub_key_file) 
       or die("Public key file $src_pub_key_file : $!");


   my @pub_key_buf = <F_PUBKEY>;
   close(F_PUBKEY);

   $st_handle = $self->{DB_HANDLE}->prepare( "UPDATE sources SET public_key = ? WHERE id_source = ?" ) || die($!);
   $st_handle->execute( "@pub_key_buf", $src_id ) || die($!);
   $st_handle->finish();
   print "Ok, source $src_id updated\n";
}


sub check_key {
    my ($self, $src_pub_key_file) = @_;

    my $gpg_home = tmpnam;

    mkdir($gpg_home, 0700)
        or die "Can't create $gpg_home : $!";

    my $gpg = new GnuPG::Interface;
    $gpg->options->hash_init( armor    => 1,
                              homedir  => $gpg_home,
                              always_trust => 1);

    my $input   = IO::Handle->new();
    my $output  = IO::Handle->new();
    my $error   = IO::Handle->new();
    my $status_fh = IO::Handle->new();
    my $handles = GnuPG::Handles->new( stdin  => $input,
                                       stdout => $output,
                                       stderr => $error,
                                       status => $status_fh,
                                       );
    
    open(KEY_FILE, "$src_pub_key_file") or die "Can't open $src_pub_key_file : $!";
    my @pub_key = <KEY_FILE>;
    close(KEY_FILE);

    my $import_pid = $gpg->import_keys( handles => $handles );
    print $input "@pub_key";
    close($input);

    waitpid($import_pid, 0);

    my $stat = <$status_fh>;
    if($stat !~ /^\[GNUPG:\] IMPORTED/) {
        my @err = <$error>;
        die "Unexpected response from GnuPG : $stat\nReturned message : @err";
    }

    $stat = <$status_fh>;
    if($stat !~ /^\[GNUPG:\] IMPORT_RES 1/) {
        my @err = <$error>;
        die "Unexpected response from GnuPG : $stat\nReturned message : @err";
    }

    unlink("$gpg_home/pubring.gpg", "$gpg_home/pubring.gpg~", "$gpg_home/random_seed", "$gpg_home/secring.gpg");
    rmdir("$gpg_home");
}


sub DESTROY {
    my $self = shift;

    $self->Disconnect;
}
##******************************************************************************
1;
